/*
	File:		UnicodeMapper.cp

	Contains:	code to install a Unicode mapping table for endpoints
	
	Written by:	David Fedor

	Copyright:	 1997 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

*/


#include <NewtonScript.h>
#include "UnicodeMapper.h"

/*
 addresses of these functions on a US GM MP2000
	GetMappingInfoProcAddress		0x00257F4C
	InstallCharEncodingProcAddress	0x00257524

on a US GM eMate 300
	GetMappingInfoProcAddress		0x002381E8
	InstallCharEncodingProcAddress	0x002377C0

 and on French GM 2000:
	GetMappingInfoProcAddress		0x00254B3C
	InstallCharEncodingProcAddress	0x00254114
*/


extern "C" Ref InstallUnicodeMapper(RefArg rcvr, RefArg fEncodingID, RefArg mapFrom, RefArg mapTo, 
									RefArg InstallCharEncodingProcAddress, RefArg GetMappingInfoProcAddress)
{
	ConvertFromUnicodeProcPtr	fromMapProc;
	ConvertToUnicodeProcPtr		toMapProc;
	TEncodingMap	*fromMapData, *toMapData;
	UShort			encodingID = (UShort)RefToInt(fEncodingID);
	
	if (!IsBinary(mapFrom))
		ThrowBadTypeWithFrameData(kNSErrNotABinaryObject, mapFrom);
	if (!IsBinary(mapTo))
		ThrowBadTypeWithFrameData(kNSErrNotABinaryObject, mapTo);
	
	// get the mapping tables and the mapping functions.
	WITH_LOCKED_BINARY(mapFrom, mapFromPtr) {
		WITH_LOCKED_BINARY(mapTo, mapToPtr) {
			// allocate space for holding encoding data
			fromMapData = (TEncodingMap*) NewPtrClear(sizeof(TEncodingMap));
			if (MemError())
				return NILREF;
			toMapData = (TEncodingMap*) NewPtrClear(sizeof(TEncodingMap));
			if (MemError()) {
				DisposePtr((Ptr)fromMapData);
				return NILREF;
			}
		
			// call ROM function to interpret the map and give us function pointers
			GetMappingInfoProcPtr GetMappingInfo = (GetMappingInfoProcPtr) RefToInt(GetMappingInfoProcAddress);
			GetMappingInfo((char*)mapFromPtr, fromMapData, (ProcPtr*)&fromMapProc);
			GetMappingInfo((char*)mapToPtr, toMapData, (ProcPtr*)&toMapProc);
		} END_WITH_LOCKED_BINARY(mapFrom)
	} END_WITH_LOCKED_BINARY(mapTo)
	
	if ((fromMapProc == ((ConvertFromUnicodeProcPtr) -1)) || (toMapProc == ((ConvertToUnicodeProcPtr) -1)))
		return NILREF;
	
	// since the function isn't in the public jumptable, have to call it directly...
	InstallCharEncodingProcPtr InstallCharEncoding = (InstallCharEncodingProcPtr) RefToInt(InstallCharEncodingProcAddress);
	InstallCharEncoding(encodingID, (Ptr)fromMapData, (Ptr)toMapData, fromMapProc, toMapProc);

	return TRUEREF;
}
